@@ -38,18 +38,7 @@ pub fn load_code_instructions(map: &mut HashMap<String, Instruction>) {
     );
     map.insert(String::from("CODE.DO"), Instruction::new(code_do));
     map.insert(String::from("CODE.DO*"), Instruction::new(code_pop_and_do));
-    map.insert(
-        String::from("CODE.DO*COUNT"),
-        Instruction::new(code_do_count),
-    );
-    map.insert(
-        String::from("CODE.DO*RANGE"),
-        Instruction::new(code_do_range),
-    );
-    map.insert(
-        String::from("CODE.DO*TIMES"),
-        Instruction::new(code_do_times),
-    );
+    map.insert(String::from("CODE.LOOP"), Instruction::new(code_loop));
     map.insert(String::from("CODE.DUP"), Instruction::new(code_dup));
     map.insert(String::from("CODE.EXTRACT"), Instruction::new(code_extract));
     map.insert(String::from("CODE.FLUSH"), Instruction::new(code_flush));
@@ -317,96 +306,35 @@ pub fn code_pop_and_do(push_state: &mut PushState, _instruction_cache: &Instruct
     }
 }
 
-/// CODE.DO*COUNT: An iteration instruction that performs a loop (the body of which is taken from
-/// the CODE stack) the number of times indicated by the INTEGER argument, pushing an index (which
-/// runs from zero to one less than the number of iterations) onto the INTEGER stack prior to each
-/// execution of the loop body. This should be implemented as a macro that expands into a call to
-/// CODE.DO*RANGE.
-/// CODE.DO*COUNT takes a single INTEGER argument (the number of times that the loop will be
-/// executed) and a single CODE argument (the body of the loop). If the provided INTEGER argument
-/// is negative or zero then this becomes a NOOP. Otherwise it expands into:
-/// ( 0 <1 - IntegerArg> CODE.QUOTE <CodeArg> CODE.DO*RANGE )
-pub fn code_do_count(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(int_arg) = push_state.int_stack.pop() {
-        if let Some(code) = push_state.code_stack.pop() {
-            if int_arg < 0 {
-                return;
-            } else {
-                let macro_item = Item::list(vec![
-                    Item::instruction("CODE.DO*RANGE".to_string()),
-                    code,
-                    Item::instruction("CODE.QUOTE".to_string()),
-                    Item::int(1 - int_arg),
-                    Item::int(0),
-                ]);
-                push_state.exec_stack.push(macro_item);
-            }
-        }
-    }
-}
-
-/// CODE.DO*RANGE: An iteration instruction that executes the top item on the CODE stack a number
-/// of times that depends on the top two integers, while also pushing the loop counter onto the
-/// INTEGER stack for possible access during the execution of the body of the loop. The top integer
-/// is the "destination index" and the second integer is the "current index." First the code and
-/// the integer arguments are saved locally and popped. Then the integers are compared. If the
-/// integers are equal then the current index is pushed onto the INTEGER stack and the code (which
-/// is the "body" of the loop) is pushed onto the EXEC stack for subsequent execution. If the
-/// integers are not equal then the current index will still be pushed onto the INTEGER stack but
-/// two items will be pushed onto the EXEC stack -- first a recursive call to CODE.DO*RANGE (with
-/// the same code and destination index, but with a current index that has been either incremented
-/// or decremented by 1 to be closer to the destination index) and then the body code. Note that
-/// the range is inclusive of both endpoints; a call with integer arguments 3 and 5 will cause its
-/// body to be executed 3 times, with the loop counter having the values 3, 4, and 5. Note also
-/// that one can specify a loop that "counts down" by providing a destination index that is less
-/// than the specified current index.
-pub fn code_do_range(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+/// CODE.LOOP: An iteration instruction that executes the top item on the EXEC stack a number
+/// of times that depends on the top two INDEX items, while also pushing the loop counter onto the
+/// INDEX stack for possible access during the execution of the body of the loop.
+/// First the code and the index arguments are saved locally and popped. Then the current and the
+/// destination field of the index are compared. If they are equal nothing happens, i.e. the
+/// index pair is just removed and the loop is terminated.
+/// If the integers are not equal then the current index will be
+/// pushed onto the INDEX stack but two items will be pushed onto the EXEC stack -- first a
+/// recursive call to EXEC.LOOP (with the same code and destination index, but with a current
+/// index that has been incremented by 1 to be closer to the destination
+/// index) and then the body code.
+pub fn code_loop(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
     if let Some(body) = push_state.code_stack.pop() {
-        if let Some(indices) = push_state.int_stack.pop_vec(2) {
-            let destination_idx = indices[1];
-            let mut current_idx = indices[0];
-            if current_idx == destination_idx {
-                push_state.int_stack.push(current_idx);
-                push_state.exec_stack.push(body);
-            } else {
-                push_state.int_stack.push(current_idx);
-                if current_idx < destination_idx {
-                    current_idx += 1;
-                } else {
-                    current_idx -= 1;
-                }
+        if let Some(index) = push_state.index_stack.copy(0) {
+            if index.current < index.destination {
                 let updated_loop = Item::list(vec![
                     body.clone(),
-                    Item::instruction("CODE.DO*RANGE".to_string()),
-                    Item::int(destination_idx),
-                    Item::int(current_idx),
+                    Item::instruction("CODE.LOOP".to_string()),
+                    Item::instruction("INDEX.INCREASE".to_string()),
                 ]);
                 push_state.exec_stack.push(updated_loop);
                 push_state.exec_stack.push(body);
+            } else {
+                push_state.index_stack.pop();
             }
         }
     }
 }
 
-/// CODE.DO*TIMES: Like CODE.DO*COUNT but does not push the loop counter. This should be
-/// implemented as a macro that expands into CODE.DO*RANGE, similarly to the implementation of
-/// CODE.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the loop
-/// body code in the call to CODE.DO*RANGE. This call to INTEGER.POP will remove the loop counter,
-/// which will have been pushed by CODE.DO*RANGE, prior to the execution of the loop body.
-pub fn code_do_times(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(int_arg) = push_state.int_stack.pop_vec(2) {
-        if let Some(code) = push_state.code_stack.pop() {
-            let macro_item = Item::list(vec![
-                Item::list(vec![code, Item::instruction("INTEGER.POP".to_string())]),
-                Item::instruction("EXEC.DO*RANGE".to_string()),
-                Item::int(int_arg[1]), // destination_idx
-                Item::int(int_arg[0]), // current_idx
-            ]);
-            push_state.exec_stack.push(macro_item);
-        }
-    }
-}
-
 /// CODE.DUP: Duplicates the top item on the CODE stack. Does not pop its argument (which, if it
 /// did, would negate the effect of the duplication!).
 pub fn code_dup(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -714,6 +642,7 @@ pub fn code_yank_dup(push_state: &mut PushState, _instruction_cache: &Instructio
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::push::index::Index;
 
     pub fn icache() -> InstructionCache {
         InstructionCache::new(vec![])
@@ -951,57 +880,24 @@ mod tests {
     }
 
     #[test]
-    fn code_do_count_unfolds_to_macro() {
+    fn code_loop_pushes_body_and_updated_loop() {
         let mut test_state = PushState::new();
         test_state.code_stack.push(Item::noop());
-        test_state.int_stack.push(3);
-        code_do_count(&mut test_state, &icache());
-        assert_eq!(
-            test_state.exec_stack.to_string(),
-            "1:List: 1:Literal(0); 2:Literal(-2); 3:InstructionMeta(CODE.QUOTE); 4:InstructionMeta(NOOP); 5:InstructionMeta(CODE.DO*RANGE);;"
-        );
+        test_state.index_stack.push(Index::new(3));
+        code_loop(&mut test_state, &icache());
+        assert_eq!(test_state.exec_stack.to_string(), "1:InstructionMeta(NOOP); 2:List: 1:InstructionMeta(INDEX.INCREASE); 2:InstructionMeta(CODE.LOOP); 3:InstructionMeta(NOOP);;");
     }
 
     #[test]
-    fn code_do_range_counts_upwards() {
+    fn code_loop_removes_index_when_terminated() {
         let mut test_state = PushState::new();
         test_state.code_stack.push(Item::noop());
-        test_state.int_stack.push(3); // Current index
-        test_state.int_stack.push(5); // Destination index
-        code_do_range(&mut test_state, &icache());
-        assert_eq!(
-            test_state.exec_stack.to_string(),
-            "1:InstructionMeta(NOOP); 2:List: 1:Literal(4); 2:Literal(5); 3:InstructionMeta(CODE.DO*RANGE); 4:InstructionMeta(NOOP);;"
-        );
-        assert_eq!(test_state.int_stack.to_string(), "1:3;");
-    }
-
-    #[test]
-    fn code_do_range_counts_downwards() {
-        let mut test_state = PushState::new();
-        test_state.code_stack.push(Item::noop());
-        test_state.int_stack.push(6); // Current index
-        test_state.int_stack.push(1); // Destination index
-        code_do_range(&mut test_state, &icache());
-        assert_eq!(
-            test_state.exec_stack.to_string(),
-            "1:InstructionMeta(NOOP); 2:List: 1:Literal(5); 2:Literal(1); 3:InstructionMeta(CODE.DO*RANGE); 4:InstructionMeta(NOOP);;"
-        );
-        assert_eq!(test_state.int_stack.to_string(), "1:6;");
-    }
-
-    #[test]
-    fn code_do_times_pops_loop_counter() {
-        let mut test_state = PushState::new();
-        test_state.code_stack.push(Item::noop());
-        test_state.int_stack.push(6); // Current index
-        test_state.int_stack.push(1); // Destination index
-        code_do_times(&mut test_state, &icache());
-        assert_eq!(
-            test_state.exec_stack.to_string(),
-            "1:List: 1:Literal(6); 2:Literal(1); 3:InstructionMeta(EXEC.DO*RANGE); 4:List: 1:InstructionMeta(INTEGER.POP); 2:InstructionMeta(NOOP);;;"
-        );
-        assert_eq!(test_state.int_stack.to_string(), "");
+        let mut test_index = Index::new(3);
+        test_index.current = 3;
+        test_state.index_stack.push(test_index);
+        code_loop(&mut test_state, &icache());
+        assert_eq!(test_state.index_stack.to_string(), "");
+        assert_eq!(test_state.exec_stack.to_string(), "");
     }
 
     #[test]
