@@ -214,6 +214,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("INTVECTOR./"),
         Instruction::new(int_vector_divide),
     );
+    map.insert(
+        String::from("INTVECTOR.CONTAINS"),
+        Instruction::new(int_vector_contains),
+    );
     map.insert(
         String::from("INTVECTOR.DEFINE"),
         Instruction::new(int_vector_define),
@@ -222,10 +226,6 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("INTVECTOR.DUP"),
         Instruction::new(int_vector_dup),
     );
-    map.insert(
-        String::from("INTVECTOR.DO*RANGE"),
-        Instruction::new(int_vector_do_range),
-    );
     map.insert(
         String::from("INTVECTOR.EQUAL"),
         Instruction::new(int_vector_equal),
@@ -740,6 +740,17 @@ pub fn int_vector_divide(push_state: &mut PushState, _instruction_cache: &Instru
     }
 }
 
+/// INTVECTOR.CONTAINS: Pushes true to the BOOLEAN stack if the top INTEGER is included in the
+/// top INTVECTOR item. This instruction acts as a NOOP if there is no INTEGER or INTVECTOR.
+/// The INTVECTOR items is not popped.
+pub fn int_vector_contains(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    if let Some(element) = push_state.int_stack.pop() {
+        if let Some(array) = push_state.int_vector_stack.get(0) {
+            push_state.bool_stack.push(array.values.contains(&element));
+        }
+    }
+}
+
 /// INTVECTOR.DEFINE: Defines the name on top of the NAME stack as an instruction that will
 /// push the top item of the INTVECTOR stack onto the EXEC stack.
 pub fn int_vector_define(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -814,26 +825,6 @@ pub fn int_vector_rand(push_state: &mut PushState, _instruction_cache: &Instruct
     }
 }
 
-/// INTVECTOR.DO*RANGE: An iteration instruction that executes the top item on the execution stack
-/// once for each element of the top INTVECTOR item. The element is pushed to the INTEGER stack
-/// before the instruction of the bodyelement is pushed to the INTEGER stack
-pub fn int_vector_do_range(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(mut remaining_elements) = push_state.int_vector_stack.pop() {
-        if let Some(body) = push_state.exec_stack.pop() {
-            if let Some(el) = remaining_elements.values.pop() {
-                let updated_loop = Item::list(vec![
-                    body.clone(),
-                    Item::instruction("INTVECTOR.DO*RANGE".to_string()),
-                    Item::intvec(remaining_elements),
-                ]);
-                push_state.int_stack.push(el);
-                push_state.exec_stack.push(updated_loop);
-                push_state.exec_stack.push(body);
-            }
-        }
-    }
-}
-
 /// INTVECTOR.SHOVE: Inserts the second INTEGER "deep" in the stack, at the position indexed by the
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn int_vector_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -1719,6 +1710,23 @@ mod tests {
         );
     }
 
+    #[test]
+    fn int_vector_contains_pushes_to_bool() {
+        let mut test_state = PushState::new();
+        test_state
+            .int_vector_stack
+            .push(IntVector::new(vec![3, 4, 1, 2]));
+        test_state.int_stack.push(4);
+        int_vector_contains(&mut test_state, &icache());
+        assert_eq!(test_state.bool_stack.pop().unwrap(), true);
+        assert_eq!(test_state.int_vector_stack.size(), 1);
+        test_state.int_stack.push(5);
+        int_vector_contains(&mut test_state, &icache());
+        assert_eq!(test_state.bool_stack.pop().unwrap(), false);
+        assert_eq!(test_state.int_vector_stack.size(), 1);
+        assert_eq!(test_state.int_stack.size(), 0);
+    }
+
     #[test]
     fn int_vector_define_creates_name_binding() {
         let mut test_state = PushState::new();
@@ -1793,23 +1801,6 @@ mod tests {
         }
     }
 
-    #[test]
-    fn int_vector_do_range_executes_for_each_element() {
-        let mut test_state = PushState::new();
-        test_state
-            .int_vector_stack
-            .push(IntVector::new(vec![1, 2, 3, 4, 5]));
-        test_state
-            .exec_stack
-            .push(Item::instruction("NOOP".to_string()));
-        int_vector_do_range(&mut test_state, &icache());
-        assert_eq!(test_state.int_vector_stack.size(), 0);
-        assert_eq!(
-            test_state.exec_stack.to_string(),
-            "1:InstructionMeta(NOOP); 2:List: 1:Literal([1,2,3,4]); 2:InstructionMeta(INTVECTOR.DO*RANGE); 3:InstructionMeta(NOOP);;"
-        );
-    }
-
     #[test]
     fn int_vector_shove_inserts_at_right_position() {
         let mut test_state = PushState::new();
